/* Default linker script, for normal executables */
/* Modelling real hardware (tc1796) with all available memories */
OUTPUT_FORMAT("elf32-tricore")
OUTPUT_ARCH(tricore)
ENTRY(_start)

/* __TC1766__ __TC13__ with Core TC1.3 */
__TRICORE_DERIVATE_MEMORY_MAP__ = 0x1796;

/* the internal code FLASH description */
__EXT_CODE_RAM_BEGIN = 0xa0000000;
__EXT_CODE_RAM_SIZE = 2048K ;

/* the internal code ram description */
__INT_CODE_RAM_BEGIN = 0xd4000000;
__INT_CODE_RAM_SIZE = 48K;

/* the internal data ram description */
__INT_DATA_RAM_BEGIN = 0xd0000000;
__INT_DATA_RAM_SIZE = 56K;

__RAM_END = __INT_DATA_RAM_BEGIN + __INT_DATA_RAM_SIZE;

/* the pcp memory description */
__PCP_CODE_RAM_BEGIN = 0xf0060000;
__PCP_CODE_RAM_SIZE = 12K;
__PCP_DATA_RAM_BEGIN = 0xf0050000;
__PCP_DATA_RAM_SIZE = 8K;

/* CACHE:
 * The TriCore TC1796 has a 2-way set associative instruction cache, 
 * with 32 sets, and each way can hold 4 double-words (256 bits)
 * => total cache size: 16 KByte;
 * replacement strategy: Least Recently Used
 * refill: critical double word first
 */

MEMORY
{
  /* ATTENTION: The program flash contains 2048K which is mapped into cached
	memory area as well as into the non-cached memory area. For easy 
	allocation, we divide it into two disjunct partitions of 1024K to ensure
	that no objects are allocated into the same physical memory cell. */
  ext_cflash_cached (rx):	org = 0x80000000, len = 1024K   /*  Program Flash with cached access (PFLASH) */    
  ext_cflash (rx):		org = 0xa0100000, len = 1024K   /*  Program Flash with non-cached access (PFLASH) */

  ext_dflash (r):		org = 0xafe00000, len = 128K    /*  Data Flash with non-cached access (DFLASH) */

  ext_sram (w):			org = 0xc0000000, len = 64K     /*  DMU external data memory (SRAM) */    
  ext_sbram (w):		org = 0xc03fc000, len = 16K     /*  DMU external stand-by data memory (SBRAM) */    

  /* The physical DMI contains 56K LDRAM and 8K DPRAM. For easy allocation, 
	we devide the LDRAM into 40K for free use, 8K for the stack and 8K
	for the context save area */
  int_dram (w):			org = 0xd0004000, len = 40K     /*  DMI internal local data RAM (LDRAM)   */
  int_dram_stack (w):		org = 0xd0002000, len = 8K      /*  DMI internal local data RAM (LDRAM)   */
  int_dram_csa (w):		org = 0xd0000000, len = 8K      /*  DMI internal local data RAM (LDRAM)   */
  int_dpram (w):		org = 0xd000e000, len = 8K	/*  DMI internal dual port RAM (DPRAM)    */

  int_cram (rx):		org = 0xd4000000, len = 47K     /*  PMI internal code scratch pad (SPRAM) */
  int_cram_sys (rx):		org = 0xd400bc00, len = 1K      /*  PMI part for trab table and inttab    */

  pcp_data (w):			org = 0xf0050000, len = 8K      /*  unused */    
  pcp_text(rx):			org = 0xf0060000, len = 12K     /*  unused */
}
/*
 * Define the sizes of the user and system stacks.
 */
__HEAP_SIZE   = DEFINED (__HEAP_SIZE) ? __HEAP_SIZE : 1k ;
__ISTACK_SIZE = DEFINED (__ISTACK_SIZE) ? __ISTACK_SIZE : 1k ;
__USTACK_SIZE = DEFINED (__USTACK_SIZE) ? __USTACK_SIZE : 6K ;
__CSA_SIZE    = DEFINED (__CSA_SIZE) ? __CSA_SIZE : 8K ;
/*
 * The heap is the memory between the top of the user stack and
 * __RAM_END (as defined above); programs can dynamically allocate
 * space in this area using malloc() and various other functions.
 * Below you can define the minimum amount of memory that the heap
 * should provide.
 */
__HEAP_MIN = DEFINED (__HEAP_MIN) ? __HEAP_MIN : 512 ;

SECTIONS
{
  /*
   * The startup code should be placed where the CPU expects it after a reset,
   * so we try to locate it first (non-volatile memory), no matter where it 
   * appears in the list of objects and libraries (note: because the wildcard 
   * pattern doesn't match directories, we'll try to find crt0.o in various 
   * (sub)directories).
   */
  .startup :
  {
    *(.startup_code)
	. = ALIGN(8);
  } > ext_cflash

  /*
   * Allocate space for absolute addressable sections; this requires that
   * "int_dram" starts at a TriCore segment (256M) and points to
   * some RAM area!  If these conditions are not met by your particular
   * hardware setup, you should either not use absolute data, or you
   * must move .zdata*,.zbss*,.bdata*,.bbss* input sections to some appropriate
   * memory area.
   */
 .zbss  (NOLOAD) :
  {
    ZBSS_BASE = . ;
    *(.zbss)
    *(.zbss.*)
    *(.gnu.linkonce.zb.*)
    *(.bbss)
    *(.bbss.*)
    . = ALIGN(8);
    ZBSS_END = . ;
  } > int_dram
  .zdata  :
  {
    ZDATA_BASE = . ;
    *(.zrodata)
    *(.zrodata.*)
    *(.zdata)
    *(.zdata.*)
    *(.gnu.linkonce.z.*)
    *(.bdata)
    *(.bdata.*)
    . = ALIGN(8);
    ZDATA_END = . ;
  } > int_dram 


  /*
   * Allocate trap and interrupt vector tables.
   */
  .traptab  :
  {
    *(.traptab)
    . = ALIGN(8) ;
  } > int_cram_sys

  .inttab  :
  {
    *(.inttab)
    . = ALIGN(8) ;
  } > int_cram_sys




  /* Special section for linking code into the Program Flash (PFLASH) 
   * of the PMU. Use the asm-directive .section to place any function here 
   */
  .text_cached  :
  {
    *(.text_cached)
    *(.text_cached.*)
    . = ALIGN(8);	
  } > ext_cflash_cached

  /* Analogous to .text.cached, this special section denotes
   * linking into program scratchpad-memory.
   */
  .text_spm : 
  {
    *(.text_spm)
    *(.text_spm.*)
    . = ALIGN(8);	
  } > int_cram

  /* By default all code is allocated in external flash memory.
   */
  .text  :
  {
    *(.text)
    *(.text)
    *(.pcp_c_ptr_init)
    *(.pcp_c_ptr_init.*)
    *(.gnu.linkonce.t.*)
    /*
     * .gnu.warning sections are handled specially by elf32.em.
     */
    *(.gnu.warning)

	. = ALIGN(8);
  } > ext_cflash



  .rodata   :
  {
    *(.rodata)
    *(.rodata.*)
    *(.gnu.linkonce.r.*)
    *(.rodata1)
    *(.toc)
    /*
     * Create the clear and copy tables that tell the startup code
     * which memory areas to clear and to copy, respectively.
     */
    . = ALIGN(4) ;
    PROVIDE(__clear_table = .) ;
    LONG(0 + ADDR(.bss));     LONG(SIZEOF(.bss));
    LONG(0 + ADDR(.sbss));    LONG(SIZEOF(.sbss));
    LONG(0 + ADDR(.zbss));    LONG(SIZEOF(.zbss));
    LONG(-1);                 LONG(-1);
    PROVIDE(__copy_table = .) ;
/*     LONG(LOADADDR(.data));    LONG(0 + ADDR(.data)); LONG(SIZEOF(.data)); */
/*     LONG(LOADADDR(.sdata));   LONG(0 + ADDR(.sdata));LONG(SIZEOF(.sdata)); */
/*     LONG(LOADADDR(.zdata));   LONG(0 + ADDR(.zdata));LONG(SIZEOF(.zdata)); */
/*     LONG(LOADADDR(.pcpdata)); LONG(0 + ADDR(.pcpdata)); LONG(SIZEOF(.pcpdata)); */
/*     LONG(LOADADDR(.pcptext)); LONG(0 + ADDR(.pcptext));LONG(SIZEOF(.pcptext)); */
    LONG(-1);                 LONG(-1);                  LONG(-1);
    . = ALIGN(8);
  } > ext_dflash
  .sdata2  :
  {
    *(.sdata.rodata)
    *(.sdata.rodata.*)
    . = ALIGN(8);
  }

  /*
   * C++ exception handling tables.  NOTE: gcc emits .eh_frame
   * sections when compiling C sources with debugging enabled (-g).
   * If you can be sure that your final application consists
   * exclusively of C objects (i.e., no C++ objects), you may use
   * the -R option of the "strip" and "objcopy" utilities to remove
   * the .eh_frame section from the executable.
   */
  .eh_frame  :
  {
    *(.gcc_except_table)
    __EH_FRAME_BEGIN__ = . ;
    *(.eh_frame)
    __EH_FRAME_END__ = . ;
	. = ALIGN(8);
  } > int_cram_sys
  /*
   * Constructors and destructors.
   */
  .ctors :
  {
    __CTOR_LIST__ = . ;
    LONG((__CTOR_END__ - __CTOR_LIST__) / 4 - 2);
    *(.ctors)
    LONG(0) ;
    __CTOR_END__ = . ;
	. = ALIGN(8);
  } > int_cram_sys
  .dtors :
  {
    __DTOR_LIST__ = . ;
    LONG((__DTOR_END__ - __DTOR_LIST__) / 4 - 2);
    *(.dtors)
    LONG(0) ;
    __DTOR_END__ = . ;
	. = ALIGN(8);
  } > int_cram_sys

  /*
   * We're done now with the text part of the executable.  The
   * following sections are special in that their initial code or
   * data (if any) must also be stored in said text part of an
   * executable, but they "live" at completely different addresses
   * at runtime -- usually in RAM areas.  NOTE: This is not really
   * necessary if you use a special program loader (e.g., a debugger)
   * to load a complete executable consisting of code, data, BSS, etc.
   * into the RAM of some target hardware or a simulator, but it *is*
   * necessary if you want to burn your application into non-volatile
   * memories such as EPROM or FLASH.
   */
  .pcptext :
  {
    PCODE_BASE = . ;
    *(.pcptext)
    *(.pcptext.*)
    . = ALIGN(8) ;
   PCODE_END = . ;
  } > pcp_text
  .pcpdata :
  {
    PRAM_BASE = . ;
    *(.pcpdata)
    *(.pcpdata.*)
    . = ALIGN(8) ;
    PRAM_END = . ;
  } > pcp_data

  .data :
  {
    . = ALIGN(8) ;
    DATA_BASE = . ;
    *(.data)
    *(.data.*)
    *(.gnu.linkonce.d.*)
    . = ALIGN(8) ;
    DATA_END = . ;
  } > int_dram

  /* Special section for linking data into internal dual port Data Memora
   * (DPRAM) of the DMI. Use the asm-directive .section to place any 
   * data-objects here.
   */
  .data_dpram :
  {
    . = ALIGN(8) ;
    *(.data_dpram)
    *(.data_dpram.*)
  } > int_dpram

  /* Special section for linking data into the Data Flash (DFLASH) 
   * of the PMU. Use the asm-directive .section to place any data-objects 
   * here.
   */
  .data_flash :
  {
    . = ALIGN(8) ;
    *(.data_flash)
    *(.data_flash.*)
  } > ext_dflash

  /* Special section for linking data into external Data Memora (SRAM) 
   * of the DMU. Use the asm-directive .section to place any data-objects 
   * here.
   */
  .data_sram :
  {
    . = ALIGN(8) ;
    *(.data_sram)
    *(.data_sram)
  } > ext_sram

  /* Special section for linking data into external Stand-by Data Memora 
   * (SBRAM) of the DMU. Use the asm-directive .section to place any 
   * data-objects here.
   */
  .data_sbram :
  {
    . = ALIGN(8) ;
    *(.data_sbram)
    *(.data_sbram.*)
  } > ext_sbram

  .sdata  :
  {
    . = ALIGN(8) ;
    SDATA_BASE = . ;
    _SMALL_DATA_ = .;
    PROVIDE(__sdata_start = .);
    *(.sdata)
    *(.sdata.*)
    *(.gnu.linkonce.s.*)
    . = ALIGN(8) ;
  } > int_dram

  .sbss  :
  {
    PROVIDE(__sbss_start = .);
    *(.sbss)
    *(.sbss.*)
    *(.gnu.linkonce.sb.*)
    . = ALIGN(8) ;
  } > int_dram
  /*
   * Allocate space for BSS sections.
   */
  .bss  (NOLOAD) :
  {
    BSS_BASE = . ;
    *(.bss)
    *(.bss.*)
    *(.gnu.linkonce.b.*)
    *(COMMON)
  } > int_dram

/*
 * Define a section to store pointer to stack.
 * We do not allocate any memory objects here!
 */
  .stack (NOLOAD) :
  {
    __HEAP = . ;
    __HEAP_END = __HEAP + __HEAP_SIZE ;
    __ISTACK = __HEAP_END + __ISTACK_SIZE ;
    __USTACK = __ISTACK + __USTACK_SIZE ;
  } > int_dram_stack


  /* define the CSA Memory area as an own section
   * this section will be allocated into the internal RAM.
  */


  .csa (NOLOAD) :
  {
    __CSA_BEGIN = . ;
    . +=  __CSA_SIZE ;
    __CSA_END = .;
  } > int_dram_csa



  _end = __HEAP_END ;
  PROVIDE(end = _end) ;
  /* Make sure CSA, stack and heap addresses are properly aligned.  */
/*   _. = ASSERT ((__CSA_BEGIN & 0x3f) == 0 , "illegal CSA start address") ; */
/*   _. = ASSERT ((__CSA_SIZE & 0x3f) == 0 , "illegal CSA size") ;	   */
/*   _. = ASSERT ((__ISTACK & 7) == 0 , "ISTACK not doubleword aligned") ; */
/*   _. = ASSERT ((__USTACK & 7) == 0 , "USTACK not doubleword aligned") ; */
/*   _. = ASSERT ((__HEAP_END & 7) == 0 , "HEAP not doubleword aligned") ; */

  /* Define a default symbol for address 0.  */
  NULL = DEFINED (NULL) ? NULL : 0 ;
  /*
   * DWARF debug sections.
   * Symbols in the DWARF debugging sections are relative to the
   * beginning of the section, so we begin them at 0.
   */
  /*
   * DWARF 1
   */
  .comment         0 : { *(.comment) }
  .debug           0 : { *(.debug) }
  .line            0 : { *(.line) }
  /*
   * GNU DWARF 1 extensions
   */
  .debug_srcinfo   0 : { *(.debug_srcinfo) }
  .debug_sfnames   0 : { *(.debug_sfnames) }
  /*
   * DWARF 1.1 and DWARF 2
   */
  .debug_aranges   0 : { *(.debug_aranges) }
  .debug_pubnames  0 : { *(.debug_pubnames) }
  /*
   * DWARF 2
   */
  .debug_info      0 : { *(.debug_info) }
  .debug_abbrev    0 : { *(.debug_abbrev) }
  .debug_line      0 : { *(.debug_line) }
  .debug_frame     0 : { *(.debug_frame) }
  .debug_str       0 : { *(.debug_str) }
  .debug_loc       0 : { *(.debug_loc) }
  .debug_macinfo   0 : { *(.debug_macinfo) }
  .debug_ranges    0 : { *(.debug_ranges) }
  /*
   * SGI/MIPS DWARF 2 extensions
   */
  .debug_weaknames 0 : { *(.debug_weaknames) }
  .debug_funcnames 0 : { *(.debug_funcnames) }
  .debug_typenames 0 : { *(.debug_typenames) }
  .debug_varnames  0 : { *(.debug_varnames) }
  /*
   * Optional sections that may only appear when relocating.
   */
  /*
   * Optional sections that may appear regardless of relocating.
   */
  .version_info    0 : { *(.version_info) }
  .boffs           0 : { *(.boffs) }
}
